
package org.w3c.tidy;

import java.util.*;

/**
 * Provides a method for non-Tidy classes to register external
 * handlers to extend the core HTML parsing.  Provides 
 * {@link NodeHandler NodeHandler}
 * implementations with access to each node as it is parsed and 
 * provides a facility for adding dynamic content.
 *
 * @author Ryan Breen  (ryan@porivo.com)
 * @see NodeHandler
 * @see Node#insertNodeAtEnd
 * @see StreamInImpl#readCharFromStream
 */
public class NodeHandlerStore {

   /**
    * Store for each registered handler.  Handlers are stored in
    * the order in which they are added.
    */
   private LinkedList handlers;

   /**
    * Singleton instance of NodeHandlerStore
    */
   private static NodeHandlerStore myInstance = 
      new NodeHandlerStore();

   /**
    * Private constructor for singleton, initializes handler list.
    */
   private NodeHandlerStore() {
      handlers = new LinkedList();
   }

   /**
    * Singleton accessor method, returns the global instance of the
    * NodeHandlerStore.
    */
   public static NodeHandlerStore getInstance() {
      return myInstance;
   }

   /**
    * Register a new NodeHandler implementation with the NodeHandlerStore.
    * NodeHandlers will be stored and queried in the order in which they
    * are added.
    * @param handler A new NodeHandler implementation to register as active.
    */
   public void addNewHandler(NodeHandler handler) {
      handlers.add(handler);
   }

   /**
    * Returns the list of registered NodeHandler implementations.
    */
   public List getHandlers() {
      return handlers;
   }

   /**
    * Submits the element and node to the handleNode method of each
    * NodeHandler implementation.  These calls are made serially with
    * each NodeHandler being queried in the order in whcih they were
    * added through calls to {@link #addNewHandler addNewHandler}.
    *
    * This call is made from the 
    * {@link Node#insertNodeAtEnd Node.insertNodeAtEnd} method, so the
    * input element and node variables represent the parent element and
    * the node that is being appended. 
    *
    * @param element Parent element for the new node element
    * @param node Node to be inserted into the DOM under the parent element
    * @return The node to be appended to the DOM, after being inspected
    * and potentially manipulated by each NodeHandler.
    */
   public org.w3c.dom.Node handleNode(org.w3c.dom.Node element, 
                                      org.w3c.dom.Node node) {
      Iterator it = handlers.iterator();
      while (it.hasNext()) {
         node = ((NodeHandler) it.next()).handleNode(element, node);
      }

      return node;
   }

   /**
    * Iterates through the registered NodeHandlers, calling the
    * {@link NodeHandler#getDynamicContent getDynamicContent} method
    * on each, until the list is exhausted or one of the handlers returns
    * a non -1 value.
    *
    * @return -1 if there is no dynamic input in any of the NodeHandlers
    * or the first byte of dynamic content available.
    */
   public int getDynamicInput() throws java.io.IOException {
      Iterator it = handlers.iterator();
      while (it.hasNext()) {
         int c = ((NodeHandler) it.next()).getDynamicContent();
         if (c != -1) {
            return c;
         }
      }
      return -1;
   }

}
